"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Determines whether a sequence contains any elements.
 * If predicate is specified, determines whether any element of a sequence satisfies a condition.
 * @param source The AsyncIterable<T> to check for emptiness or apply the predicate to.
 * @param predicate A function to test each element for a condition.
 * @returns ``true`` if every element of the source sequence passes the test in the specified predicate,
 * or if the sequence is empty; otherwise, ``false``.
 */
function any(source, predicate) {
    if (predicate) {
        return any2(source, predicate);
    }
    else {
        return any1(source);
    }
}
exports.any = any;
const any1 = async (source) => {
    var e_1, _a;
    try {
        for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = await source_1.next(), !source_1_1.done;) {
            const _ = source_1_1.value;
            return true;
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (source_1_1 && !source_1_1.done && (_a = source_1.return)) await _a.call(source_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return false;
};
const any2 = async (source, predicate) => {
    var e_2, _a;
    try {
        for (var source_2 = tslib_1.__asyncValues(source), source_2_1; source_2_1 = await source_2.next(), !source_2_1.done;) {
            const item = source_2_1.value;
            if (predicate(item) === true) {
                return true;
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (source_2_1 && !source_2_1.done && (_a = source_2.return)) await _a.call(source_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return false;
};
