"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Returns distinct elements from a sequence by using the specified equality comparer to compare values.
 * @param source The sequence to remove duplicate elements from.
 * @param comparer An IAsyncEqualityComparer<T> to compare values.
 * @returns An IAsyncEnumerable<T> that contains distinct elements from the source sequence.
 */
function distinctAsync(source, comparer) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a;
            const distinctElements = [];
            try {
                outerLoop: for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                    const item = source_1_1.value;
                    for (const distinctElement of distinctElements) {
                        const found = yield tslib_1.__await(comparer(distinctElement, item));
                        if (found) {
                            continue outerLoop;
                        }
                    }
                    distinctElements.push(item);
                    yield yield tslib_1.__await(item);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
}
exports.distinctAsync = distinctAsync;
