"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Produces the set difference of two sequences by using the comparer provided to compare values.
 * @param first An AsyncIterable<T> whose elements that are not also in second will be returned.
 * @param second An AsyncIterable<T> whose elements that also occur in the first sequence
 * will cause those elements to be removed from the returned sequence.
 * @param comparer An IAsyncEqualityComparer<T> to compare values.
 * @returns A sequence that contains the set difference of the elements of two sequences.
 */
function exceptAsync(first, second, comparer) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a, e_2, _b;
            // TODO: async eq of [...second] ?
            const secondArray = [];
            try {
                for (var second_1 = tslib_1.__asyncValues(second), second_1_1; second_1_1 = yield tslib_1.__await(second_1.next()), !second_1_1.done;) {
                    const x = second_1_1.value;
                    secondArray.push(x);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (second_1_1 && !second_1_1.done && (_a = second_1.return)) yield tslib_1.__await(_a.call(second_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                for (var first_1 = tslib_1.__asyncValues(first), first_1_1; first_1_1 = yield tslib_1.__await(first_1.next()), !first_1_1.done;) {
                    const firstItem = first_1_1.value;
                    let exists = false;
                    for (let j = 0; j < secondArray.length; j++) {
                        const secondItem = secondArray[j];
                        if ((yield tslib_1.__await(comparer(firstItem, secondItem))) === true) {
                            exists = true;
                            break;
                        }
                    }
                    if (exists === false) {
                        yield yield tslib_1.__await(firstItem);
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (first_1_1 && !first_1_1.done && (_b = first_1.return)) yield tslib_1.__await(_b.call(first_1));
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
}
exports.exceptAsync = exceptAsync;
