"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ErrorString_1 = require("../../shared/ErrorString");
const InvalidOperationException_1 = require("../../shared/InvalidOperationException");
/**
 * Returns the first element of a sequence.
 * If predicate is specified, returns the first element in a sequence that satisfies a specified condition.
 * @param source The AsyncIterable<T> to return the first element of.
 * @param predicate A function to test each element for a condition. Optional.
 * @throws {InvalidOperationException} The source sequence is empty.
 * @returns The first element in the specified sequence.
 * If predicate is specified,
 * the first element in the sequence that passes the test in the specified predicate function.
 */
function first(source, predicate) {
    if (predicate) {
        return first2(source, predicate);
    }
    else {
        return first1(source);
    }
}
exports.first = first;
const first1 = async (source) => {
    const firstElement = await source[Symbol.asyncIterator]().next();
    if (firstElement.done === true) {
        throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
    }
    return firstElement.value;
};
const first2 = async (source, predicate) => {
    var e_1, _a;
    try {
        for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = await source_1.next(), !source_1_1.done;) {
            const value = source_1_1.value;
            if (predicate(value) === true) {
                return value;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (source_1_1 && !source_1_1.done && (_a = source_1.return)) await _a.call(source_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoMatch);
};
