"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Returns first element in sequence that satisfies predicate otherwise
 * returns the first element in the sequence. Returns null if no value found.
 * @param source An AsyncIterable<T> to return an element from.
 * @param predicate A function to test each element for a condition. Optional.
 * @returns The first element in the sequence
 * or the first element that passes the test in the specified predicate function.
 * Returns null if no value found.
 */
function firstOrDefault(source, predicate) {
    if (predicate) {
        return firstOrDefault2(source, predicate);
    }
    else {
        return firstOrDefault1(source);
    }
}
exports.firstOrDefault = firstOrDefault;
const firstOrDefault1 = async (source) => {
    const first = await source[Symbol.asyncIterator]().next();
    return first.value || null;
};
const firstOrDefault2 = async (source, predicate) => {
    var e_1, _a;
    try {
        for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = await source_1.next(), !source_1_1.done;) {
            const value = source_1_1.value;
            if (predicate(value) === true) {
                return value;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (source_1_1 && !source_1_1.done && (_a = source_1.return)) await _a.call(source_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return null;
};
