"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ArgumentOutOfRangeException_1 = require("../../shared/ArgumentOutOfRangeException");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Generates a sequence that contains one repeated value.
 * @param element The value to be repeated.
 * @param count The number of times to repeat the value in the generated sequence.
 * @returns An IAsyncEnumerable<T> that contains a repeated value.
 */
function repeat(element, count, delay) {
    if (count < 0) {
        throw new ArgumentOutOfRangeException_1.ArgumentOutOfRangeException(`count`);
    }
    if (delay) {
        return repeat2(element, count, delay);
    }
    else {
        return repeat1(element, count);
    }
}
exports.repeat = repeat;
const repeat1 = (element, count) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            for (let i = 0; i < count; i++) {
                yield yield tslib_1.__await(element);
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
};
const repeat2 = (element, count, delay) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_2() {
            for (let i = 0; i < count; i++) {
                yield yield tslib_1.__await(yield tslib_1.__await(new Promise((resolve) => setTimeout(() => resolve(element), delay))));
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
};
