"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ErrorString_1 = require("../../shared/ErrorString");
const InvalidOperationException_1 = require("../../shared/InvalidOperationException");
/**
 * Returns the only element of a sequence that satisfies a specified condition (if specified),
 * and throws an exception if more than one such element exists.
 * @param source An AsyncIterable<T> to return a single element from.
 * @param predicate A function to test an element for a condition. (Optional)
 * @throws {InvalidOperationException} No element satisfies the condition in predicate. OR
 * More than one element satisfies the condition in predicate. OR
 * The source sequence is empty.
 * @returns The single element of the input sequence that satisfies a condition.
 */
function single(source, predicate) {
    if (predicate) {
        return single2(source, predicate);
    }
    else {
        return single1(source);
    }
}
exports.single = single;
const single1 = async (source) => {
    var e_1, _a;
    let hasValue = false;
    let singleValue = null;
    try {
        for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = await source_1.next(), !source_1_1.done;) {
            const value = source_1_1.value;
            if (hasValue === true) {
                throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.MoreThanOneElement);
            }
            else {
                hasValue = true;
                singleValue = value;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (source_1_1 && !source_1_1.done && (_a = source_1.return)) await _a.call(source_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    if (hasValue === false) {
        throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
    }
    return singleValue;
};
const single2 = async (source, predicate) => {
    var e_2, _a;
    let hasValue = false;
    let singleValue = null;
    try {
        for (var source_2 = tslib_1.__asyncValues(source), source_2_1; source_2_1 = await source_2.next(), !source_2_1.done;) {
            const value = source_2_1.value;
            if (predicate(value)) {
                if (hasValue === true) {
                    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.MoreThanOneMatchingElement);
                }
                else {
                    hasValue = true;
                    singleValue = value;
                }
            }
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (source_2_1 && !source_2_1.done && (_a = source_2.return)) await _a.call(source_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    if (hasValue === false) {
        throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoMatch);
    }
    return singleValue;
};
