"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Produces the set union of two sequences by using scrict equality comparison or a specified IEqualityComparer<T>.
 * @param first An AsyncIterable<T> whose distinct elements form the first set for the union.
 * @param second An AsyncIterable<T> whose distinct elements form the second set for the union.
 * @param comparer The IEqualityComparer<T> to compare values. Optional.
 * @returns An IAsyncEnumerable<T> that contains the elements from both input sequences, excluding duplicates.
 */
function union(first, second, comparer) {
    if (comparer) {
        return union2(first, second, comparer);
    }
    else {
        return union1(first, second);
    }
}
exports.union = union;
const union1 = (first, second) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a, e_2, _b;
            const set = new Set();
            try {
                for (var first_1 = tslib_1.__asyncValues(first), first_1_1; first_1_1 = yield tslib_1.__await(first_1.next()), !first_1_1.done;) {
                    const item = first_1_1.value;
                    if (set.has(item) === false) {
                        yield yield tslib_1.__await(item);
                        set.add(item);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (first_1_1 && !first_1_1.done && (_a = first_1.return)) yield tslib_1.__await(_a.call(first_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                for (var second_1 = tslib_1.__asyncValues(second), second_1_1; second_1_1 = yield tslib_1.__await(second_1.next()), !second_1_1.done;) {
                    const item = second_1_1.value;
                    if (set.has(item) === false) {
                        yield yield tslib_1.__await(item);
                        set.add(item);
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (second_1_1 && !second_1_1.done && (_b = second_1.return)) yield tslib_1.__await(_b.call(second_1));
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
};
const union2 = (first, second, comparer) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_2() {
            var e_3, _a;
            const result = [];
            for (const source of [first, second]) {
                try {
                    for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                        const value = source_1_1.value;
                        let exists = false;
                        for (const resultValue of result) {
                            if (comparer(value, resultValue) === true) {
                                exists = true;
                                break;
                            }
                        }
                        if (exists === false) {
                            yield yield tslib_1.__await(value);
                            result.push(value);
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
};
