"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorString_1 = require("../../shared/ErrorString");
const InvalidOperationException_1 = require("../../shared/InvalidOperationException");
const toArray_1 = require("./toArray");
/**
 * Returns the first element in a sequence that satisfies a specified condition.
 * @param source An IParallelEnumerable<T> to return an element from.
 * @param predicate An async function to test each element for a condition.
 * @throws {InvalidOperationException} No elements in Iteration matching predicate
 * @returns The first element in the sequence that passes the test in the specified predicate function.
 */
async function firstAsync(source, predicate) {
    const data = await toArray_1.toArray(source);
    for (const value of data) {
        if (await predicate(value) === true) {
            return value;
        }
    }
    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoMatch);
}
exports.firstAsync = firstAsync;
