"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const StrictEqualityComparer_1 = require("../../shared/StrictEqualityComparer");
/**
 * Compares two parallel sequences to see if they are equal using a comparer function.
 * @param first First Sequence
 * @param second Second Sequence
 * @param comparer Comparer
 * @returns Whether or not the two iterations are equal
 */
async function sequenceEquals(
// tslint:disable-next-line:no-shadowed-variable
first, second, comparer = StrictEqualityComparer_1.StrictEqualityComparer) {
    const firstArray = await first.toArray();
    const secondArray = await second.toArray();
    if (firstArray.length !== secondArray.length) {
        return false;
    }
    for (let i = 0; i < firstArray.length; i++) {
        const firstResult = firstArray[i];
        const secondResult = secondArray[i];
        if (comparer(firstResult, secondResult) === false) {
            return false;
        }
    }
    return true;
}
exports.sequenceEquals = sequenceEquals;
