"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Returns distinct elements from a sequence by using the specified equality comparer to compare values.
 * @param source The sequence to remove duplicate elements from.
 * @param comparer An IAsyncEqualityComparer<T> to compare values.
 * @returns An IAsyncEnumerable<T> that contains distinct elements from the source sequence.
 */
function distinctAsync(source, comparer) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            const distinctElements = [];
            outerLoop: for (const item of source) {
                for (const distinctElement of distinctElements) {
                    const found = yield tslib_1.__await(comparer(distinctElement, item));
                    if (found) {
                        continue outerLoop;
                    }
                }
                distinctElements.push(item);
                yield yield tslib_1.__await(item);
            }
        });
    }
    return from_1.from(iterator);
}
exports.distinctAsync = distinctAsync;
