"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
const Grouping_1 = require("../Grouping");
function groupByAsync(source, keySelector, comparer) {
    if (comparer) {
        return groupByAsync_0(source, keySelector, comparer);
    }
    else {
        return groupByAsync_0_Simple(source, keySelector);
    }
}
exports.groupByAsync = groupByAsync;
function groupByAsync_0_Simple(source, keySelector) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            const keyMap = {};
            for (const value of source) {
                const key = yield tslib_1.__await(keySelector(value));
                const grouping = keyMap[key];
                if (grouping) {
                    grouping.push(value);
                }
                else {
                    keyMap[key] = new Grouping_1.Grouping(key, value);
                }
            }
            // tslint:disable-next-line:forin
            for (const value in keyMap) {
                yield yield tslib_1.__await(keyMap[value]);
            }
        });
    }
    return from_1.from(iterator);
}
function groupByAsync_0(source, keySelector, comparer) {
    function generate() {
        return tslib_1.__asyncGenerator(this, arguments, function* generate_1() {
            const keyMap = new Array();
            for (const value of source) {
                const key = yield tslib_1.__await(keySelector(value));
                let found = false;
                for (let i = 0; i < keyMap.length; i++) {
                    const group = keyMap[i];
                    if ((yield tslib_1.__await(comparer(group.key, key))) === true) {
                        group.push(value);
                        found = true;
                        break;
                    }
                }
                if (found === false) {
                    keyMap.push(new Grouping_1.Grouping(key, value));
                }
            }
            for (const keyValue of keyMap) {
                yield yield tslib_1.__await(keyValue);
            }
        });
    }
    return from_1.from(generate);
}
