"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Produces the set intersection of two sequences by using the specified IAsyncEqualityComparer<T> to compare values.
 * @param first An IEnumerable<T> whose distinct elements that also appear in second will be returned.
 * @param second An Iterable<T> whose distinct elements that also appear in the first sequence will be returned.
 * @param comparer An IAsyncEqualityComparer<T> to compare values.
 * @returns A sequence that contains the elements that form the set intersection of two sequences.
 */
function intersectAsync(first, second, comparer) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a;
            const firstResults = [];
            try {
                for (var _b = tslib_1.__asyncValues(first.distinctAsync(comparer)), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                    const item = _c.value;
                    firstResults.push(item);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (firstResults.length === 0) {
                return yield tslib_1.__await(void 0);
            }
            const secondResults = [...second];
            for (let i = 0; i < firstResults.length; i++) {
                const firstValue = firstResults[i];
                for (let j = 0; j < secondResults.length; j++) {
                    const secondValue = secondResults[j];
                    if ((yield tslib_1.__await(comparer(firstValue, secondValue))) === true) {
                        yield yield tslib_1.__await(firstValue);
                        break;
                    }
                }
            }
        });
    }
    return from_1.from(iterator);
}
exports.intersectAsync = intersectAsync;
