"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Bypasses elements in a sequence as long as a specified condition is true and then returns the remaining elements.
 * The element's index is used in the logic of the predicate function.
 * @param source An Iterable<T> to return elements from.
 * @param predicate A function to test each source element for a condition;
 * the second parameter of the function represents the index of the source element.
 * @returns An IAsyncEnumerable<T> that contains the elements from the input sequence starting
 * at the first element in the linear series that does not pass the test specified by predicate.
 */
function skipWhileAsync(source, predicate) {
    if (predicate.length === 1) {
        return skipWhileAsync1(source, predicate);
    }
    else {
        return skipWhileAsync2(source, predicate);
    }
}
exports.skipWhileAsync = skipWhileAsync;
const skipWhileAsync1 = (source, predicate) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            let skip = true;
            for (const item of source) {
                if (skip === false) {
                    yield yield tslib_1.__await(item);
                }
                else if ((yield tslib_1.__await(predicate(item))) === false) {
                    skip = false;
                    yield yield tslib_1.__await(item);
                }
            }
        });
    }
    return from_1.from(iterator);
};
const skipWhileAsync2 = (source, predicate) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_2() {
            let index = 0;
            let skip = true;
            for (const item of source) {
                if (skip === false) {
                    yield yield tslib_1.__await(item);
                }
                else if ((yield tslib_1.__await(predicate(item, index))) === false) {
                    skip = false;
                    yield yield tslib_1.__await(item);
                }
                index++;
            }
        });
    }
    return from_1.from(iterator);
};
