"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Returns elements from a sequence as long as a specified condition is true.
 * The element's index is used in the logic of the predicate function.
 * @param source The sequence to return elements from.
 * @param predicate A async function to test each source element for a condition;
 * the second parameter of the function represents the index of the source element.
 * @returns An IAsyncEnumerable<T> that contains elements from the input sequence
 * that occur before the element at which the test no longer passes.
 */
function takeWhileAsync(source, predicate) {
    if (predicate.length === 1) {
        return takeWhileAsync1(source, predicate);
    }
    else {
        return takeWhileAsync2(source, predicate);
    }
}
exports.takeWhileAsync = takeWhileAsync;
const takeWhileAsync1 = (source, predicate) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            for (const item of source) {
                if (yield tslib_1.__await(predicate(item))) {
                    yield yield tslib_1.__await(item);
                }
                else {
                    break;
                }
            }
        });
    }
    return from_1.from(iterator);
};
const takeWhileAsync2 = (source, predicate) => {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_2() {
            let index = 0;
            for (const item of source) {
                if (yield tslib_1.__await(predicate(item, index++))) {
                    yield yield tslib_1.__await(item);
                }
                else {
                    break;
                }
            }
        });
    }
    return from_1.from(iterator);
};
